#ifndef LINT
static char ephem_sid[] = "@(#) $Id: ephem.c,v 2.0 2003/06/06 20:40:49 bolding Exp $";
#endif

/*
 * ephem.c - ephemeredes for elist tool
 */

#include "elist.h"
#include "elist_ext.h"

/*
 * idoy - returns the day number of a date, relative to 1st of that year
 */
   int idoy(iyr, imo, idy)
      int iyr, imo, idy;

      {
         return(julian(imo, idy, iyr) - julian(1, 1, iyr) + 1);
      }

/*
 * calcL0 - calculate and return L0 value
 *
 * based on ephemeris for the sun code by Frank Hill.
 *
 * formulae are from 1985 nautical almanac, p.c24,
 * input: ints iyr (4 digit year)
 *                 idy (day number)
 *                 ihr (UT hour)
 *                 imn (UT min)
 *                 isc (UT sec)
 * output (all real *8)
 *   bo: b angle, heliographic latitude of disk center in degrees 
 *   alo: l angle, heliographic longitude of disk center in degrees 
 *   p:   p angle, position angle of axis in degrees 
 *   radius: apparent semi-diameter in arcsec 
 *   ra: right ascension in degrees 
 *   dec: declination in degrees 
 *   gmsta: greenwich mean sidereal time at 0 ut in hours 
 *   gmstn: greenwich mean sidereal time at input time 
 *   gha: greenwich hour angle at input time in degrees 
 */

   double calcL0(iyr, idy, ihr, imn, isc)
      int iyr, idy, ihr, imn, isc;

      {
         /* Initialized data */

         double bo, alo, p, radius, ra, dec, gmsta, gmstn, gha;
         int lquad();

         static double ajd[55] =
            { 25567.,25932.,26297.,26663.,27028.,27393.,
            27757.,28124.,28489.,28854.,29219.,29585.,29950.,30315.,30680.,
            31046.,31411.,31776.,32141.,32507.,32872.,33237.,33602.,33968.,
            34333.,34698.,35063.,35429.,35794.,36159.,36524.,36890.,37255.,
            37620.,37985.,38351.,38716.,39081.,39446.,39812.,40177.,40542.,
            40907.,41273.,41638.,42003.,42368.,42734.,43099.,43464.,43829.,
            44195.,44560.,44925.,45290. };
         static double pi = 3.14159265358979323846;

         /* Local variables */
         static double ainc, grad;
         static double gmst, d__, g, q, r__, t, omega;
         static int igmst;
         static double d0, al, am, an, el;
         static int iq;
         static double tn, tu, an0, tu2, tu3, rad, eps, eot, alo1, alo2;

         rad = pi / 180.;
         if (iyr < 1970 || iyr > 2024) {
            fprintf(stderr, "Ephemeris from 1970 to 2024 only.\n");
            return(-1);
            }

         ainc = 7.25;
         d__ = ajd[iyr - 1970] + 2415019.5 + idy + ihr / 24. + imn /
            1440. + isc / (float)86400.;
         d0 = ajd[iyr - 1970] + 2415019.5 + idy;
         t = (d__ - 2396757.5) / 365.242191;
         omega = t * .013958333333333333 + 73.666666666666671;
         an = d__ - 2451545.;
         an0 = d0 - 2451545.;
         al = an * .9856474 + 280.46;
         g = an * .9856003 + 357.528;
         lquad(&al);
         lquad(&g);
         grad = g * rad;
         el = al + sin(grad) * 1.915 + sin(grad * 2.) * .02;
         lquad(&el);
         r__ = 1.00014 - cos(grad) * .01671 - cos(grad * 2.) * 1.4e-4;
         radius = 959.75999999999999 / r__;
         eps = 23.439 - an * 4e-7;
         ra = atan(cos(eps * rad) * tan(el * rad));
         ra /= rad;
         if (el >= 90. && el < 270.) {
            ra += 180.;
            }
         if (el >= 270. && el < 360.) {
            ra += 360.;
            }
         dec = asin(sin(eps * rad) * sin(el * rad));
         dec /= rad;
         eot = (al - ra) * 4.;
         am = 360. - (d__ - 2398220.) * 14.184397163120568;
         lquad(&am);
         q = el - omega;
         bo = asin(sin(q * rad) * sin(ainc * rad));
         p = atan(-cos(el * rad) * tan(eps * rad)) +
             atan(-cos(q * rad) * tan(ainc * rad));
         lquad(&q);
         if (q >= 0. && q < 90.){
            q += 180.;
            iq = 1;
            goto L10;
            }
         if (q >= 90. && q < 180.){
            iq = 2;
            }
         if (q >= 180. && q < 270.){
            iq = 3;
            }
         if (q >= 270. && q < 360.){
            iq = 4;
            }
      L10:
         alo1 = acos(-cos(q * rad) / cos(bo)) + am * rad;
         alo2 = asin(-sin(q * rad) * cos(ainc * rad) / cos(bo)) + am * rad;
         bo /= rad;
         p /= rad;
         alo = alo1 / rad;
         if (iq == 1){
            alo += 180.;
            }
         if (iq == 2){
            alo = alo2 / rad;
            }
         lquad(&alo);
         tu = an0 / 36525.;
         tu2 = tu * tu;
         tu3 = tu2 * tu;
         gmst = tu * 8640184.812866 + 24110.54841 +
            tu2 * .093104 - tu3 * 6.2e-6;
         gmst /= 3600.;
         igmst = (int) (gmst / 24.);
         gmsta = gmst - igmst * 24.;
         if (gmsta < 0.){
            gmsta += 24.;
            }
         tn = (double) (ihr) + (double) (imn) / 60. +
            (double) (isc) / 3600.;
         gmstn = gmsta + tn * 1.00273791;
         if (gmstn < 0.){
            gmstn += 24.;
            }
         if (gmstn >= 24.) {
            gmstn += -24.;
            }
         gha = gmstn * 15. - ra;
         if (gha < 0.){
            gha += 360.;
            }
         if (gha >= 360.){
            gha += -360.;
            }
         return(alo);
      }

   int lquad(x)
      double *x;
      
      {
         
         /* put the value in the right quadrant */

         LQ1:
             if (*x > 360.){
                *x += -360.;
                goto LQ1;
                }
         LQ10:
             if (*x < (float)0.){
                *x += 360.;
                goto LQ10;
                }
         return(0);
      }
